from panda3d.core import *
from direct.showbase.DirectObject import DirectObject
from direct.interval.IntervalGlobal import *

class CameraControler (DirectObject):
    def __init__(self, app, pos=(0,0,0), offset=(0, -12, 3), speed=10.0, angle=-10):
        self.app=app
        base.win.movePointer(0, base.win.getXSize()/2, base.win.getYSize()/2)
        self.cameraNode  = render.attachNewNode("cameraNode")
        self.cameraNode.setPos(pos)
        self.cameraGimbal  = self.cameraNode.attachNewNode("cameraGimbal")
        base.camera.setPos(pos)
        base.camera.setX(base.camera, offset[0])
        base.camera.setY(base.camera, offset[1])
        base.camera.setZ(offset[2])
        base.camera.setP(angle)
        base.camera.wrtReparentTo(self.cameraGimbal)
                
        self.mouseSpeed=speed
        
        taskMgr.add(self.update, "camcon_update", sort=45)
        
    def zoom(self, t):
        base.camera.setY(base.camera, t)
    
    def zoom_control(self, amount):  
        LerpFunc(self.zoom,fromData=0,toData=amount, duration=.2, blendType='easeOut').start()
        
    def _moveCamX(self, t):
        self.cameraNode.setX(self.cameraNode, t*self.mouseSpeed)
    def _moveCamY(self, t):
        self.cameraNode.setY(self.cameraNode, t*self.mouseSpeed)   
        
    def _rotateCamH(self, t):
        self.cameraNode.setH(self.cameraNode.getH()- t*self.mouseSpeed)
        
    def _rotateCamP(self, t):         
        self.cameraGimbal.setP(self.cameraGimbal.getP()+ t*self.mouseSpeed*0.8)       
        if self.cameraGimbal.getP()<-15.0:
            self.cameraGimbal.setP(-15.0)
        if self.cameraGimbal.getP()>15.0:
            self.cameraGimbal.setP(15.0)  
            
    def move_control(self, x, y):
        LerpFunc(self._moveCamX,fromData=0,toData=x, duration=.1).start()
        LerpFunc(self._moveCamY,fromData=0,toData=y, duration=.1).start()       
    
    def rotate_control(self, h, p):
        LerpFunc(self._rotateCamH,fromData=0,toData=h, duration=.3).start()
        LerpFunc(self._rotateCamP,fromData=0,toData=p, duration=.3).start()   
        
    def update(self, task): 
        if not self.app.pause:      
            if base.mouseWatcherNode.hasMouse():
                deltaX = base.mouseWatcherNode.getMouseX()  # mouse distance from the center
                deltaY = base.mouseWatcherNode.getMouseY()
                base.win.movePointer(0, base.win.getXSize()/2, base.win.getYSize()/2)            
                if abs(deltaX)+abs(deltaY)>0.001:
                    self.rotate_control(deltaX, deltaY)            
        return task.cont 
