﻿from panda3d.core import *
from direct.gui.DirectGui import *
from direct.interval.IntervalGlobal import *
from direct.showbase.DirectObject import DirectObject

def _pos2d(x,y):
    return Point3(x,0,-y)
    
def _rec2d(width, height):
    return (-width, 0, 0, height)
    
def _resetPivot(frame):
    size=frame['frameSize']    
    frame.setPos(-size[0], 0, -size[3])        
    frame.flattenLight()

MODE_MAIN=0
MODE_OPTION=1
MODE_MOUSE=2
MODE_MUSIC=3
MODE_SOUND=4
MODE_HELP=5
MODE_GAMEOVER=6

    
class GuiManager():
    def __init__(self, app): 
        #reference to the main app for function calls
        self.app=app
        
        #load font + big font
        #segoe print
        self.font = loader.loadFont('font/font.ttf')
        self.font.setPixelsPerUnit(32)
        self.font.setMinfilter(Texture.FTNearest )
        self.font.setMagfilter(Texture.FTNearest )
        self.font_big=self.font.makeCopy()        
        self.font_big.setPixelsPerUnit(42)
        self.font_big.set_outline((0, 0, 0, 0.5), 1, 0.4)

        #set nodes for gui placement
        self.top_left=pixel2d.attachNewNode('TopLeft')
        self.top_right=pixel2d.attachNewNode('TopRight')
        self.bottom_right=pixel2d.attachNewNode('BottomRight')
        self.bottom_left=pixel2d.attachNewNode('BottomLeft')
        self.top=pixel2d.attachNewNode('Top')
        self.bottom=pixel2d.attachNewNode('Bottom')
        self.left=pixel2d.attachNewNode('Left')
        self.right=pixel2d.attachNewNode('Right')
        self.center=pixel2d.attachNewNode('Center')
        
        
        
        #some vars...
        self.last_event=None        
                
        #menu       
        self.menu_frame=DirectFrame(frameSize=_rec2d(512,512),
                                frameColor=(1,0,0, 0.0),
                                parent=self.bottom)
        _resetPivot(self.menu_frame)
        self.menu_frame.setPos(_pos2d(-256, -354))
        self.menu_mode=0
        self.anim_sq=None
        self.shouldExit=False
        #label 
        self.menu_txt=[
                       ['MENU:', 'Start!', 'Options...', 'Help', 'Exit'],
                       ['OPTIONS:', 'Mouse Speed', 'Music Volume', 'Sounds Volume', '<-Back'] ,                      
                       ['MOUSE SPEED:', 'Speed: 10', '+', '-', '<-Back'] ,
                       ['MUSIC:', 'Volume: 50', '+', '-', '<-Back'] ,
                       ['SOUNDS:', 'Volume: 100', '+', '-', '<-Back'], 
                       ['', '', '', '', '<-Back'],
                       ['', '', '', '', 'Exit'],
                       ]
        self.menu_cmd=[
                       [None, (self.startGame, None), (self.setMenuMode, MODE_OPTION), (self.showHelp, None), (self.exit, None)],
                       [None, (self.setMenuMode, MODE_MOUSE), (self.setMenuMode, MODE_MUSIC), (self.setMenuMode, MODE_SOUND), (self.setMenuMode, MODE_MAIN)],
                       [None, None, (self.setMouseSpeed, 1), (self.setMouseSpeed, -1), (self.setMenuMode, MODE_OPTION)],
                       [None, None, (self.setMusicVolume, 1), (self.setMusicVolume, -1), (self.setMenuMode, MODE_OPTION)],
                       [None, None, (self.setSoundVolume, 1), (self.setSoundVolume, -1), (self.setMenuMode, MODE_OPTION)],
                       [None, None, None, None, (self.setMenuMode, MODE_MAIN)],
                       [None, None, None, None, (self.exit, None)]
                       ]        
        self.menu_buttons=[]
        for i in range(5):
            self.menu_buttons.append(DirectFrame(frameSize=_rec2d(512,64),
                                    frameColor=(1,1,1,0.99),
                                    frameTexture='icon/bar.png',                                    
                                    text=self.menu_txt[0][i],
                                    text_scale=42,
                                    text_font=self.font_big,
                                    text_align=TextNode.ACenter,
                                    text_pos=(-256,18),
                                    text_fg=(1,1,1,0.9),
                                    state=DGG.NORMAL, 
                                    parent=self.menu_frame))
            _resetPivot(self.menu_buttons[i])   
            self.menu_buttons[i].setPos(_pos2d(0,72*i))
            self.menu_buttons[i].bind(DGG.B1PRESS, self.onMenuClick, [i])
        #logo
        self.logo=DirectFrame(frameSize=_rec2d(512,256),
                                frameColor=(1,1,1,0.99),
                                frameTexture='icon/logo.png', 
                                parent=self.top)  
        _resetPivot(self.logo) 
        self.logo.setPos(_pos2d(-256,0))
        
        #timer
        self.timer=DirectFrame(frameSize=_rec2d(256,128),
                                frameColor=(1,1,1,0.99),
                                frameTexture='icon/back.png',
                                text="Time: 30.00",
                                text_scale=32,
                                text_font=self.font,
                                text_align=TextNode.ALeft,
                                text_pos=(-226,20),
                                text_fg=(1,1,1,0.9), 
                                parent=self.top_right)  
        _resetPivot(self.timer) 
        self.timer.setPos(_pos2d(-220,-80))                        
        #score
        self.data=DirectFrame(frameSize=_rec2d(256,128),
                                frameColor=(1,1,1,0.99),
                                frameTexture='icon/back.png',
                                text="0 kB",
                                text_scale=32,
                                text_font=self.font,
                                text_align=TextNode.ALeft,
                                text_pos=(-220,20),
                                text_fg=(1,1,1,0.9), 
                                parent=self.top_left)  
        _resetPivot(self.data) 
        self.data.setPos(_pos2d(-36,-80))  
        #firewall  
        self.firewall=DirectFrame(frameSize=_rec2d(256,128),
                                frameColor=(1,1,1,0.99),
                                frameTexture='icon/firewall.png',                                
                                parent=self.top)  
        _resetPivot(self.firewall) 
        self.firewall.setPos(_pos2d(-128,-70)) 
        self.firewall.hide()
        
        #gameover
        self.gameover=DirectFrame(frameSize=_rec2d(512,512),
                                frameColor=(1,1,1,0.99),
                                frameTexture='icon/game_over.png',                                
                                parent=self.center)  
        _resetPivot(self.gameover) 
        self.gameover.setPos(_pos2d(-256,-256)) 
        self.gameover.hide()                            
        #help
        self.help=DirectFrame(frameSize=_rec2d(512,512),
                                frameColor=(1,1,1,0.99),
                                frameTexture='icon/help.png',                                
                                parent=self.center)  
        _resetPivot(self.help) 
        self.help.setPos(_pos2d(-256,-256)) 
        self.help.hide()     
        
                                                
        self.updateGuiNodes()                             
        #tasks
        taskMgr.add(self.update, 'GuiManager_update')
    
    def onMenuClick(self, button, event=None):  
        if self.menu_cmd[self.menu_mode][button]:
            self.last_event=event
            old_pos=self.menu_buttons[button].getPos()
            self.menu_buttons[button].setPos(old_pos+_pos2d(0,5))            
            cmd=self.menu_cmd[self.menu_mode][button][0]
            arg=self.menu_cmd[self.menu_mode][button][1]
            #cmd(arg)
            self.anim_sq=Sequence(Wait(0.05), LerpPosInterval(self.menu_buttons[button], 0.08,old_pos),Wait(0.1), Func(cmd, arg))
            self.anim_sq.start()
            
    def exit(self, arg=None): 
        self.shouldExit=True                
    
    def doGameOver(self):
        self.gameover.show() 
        self.menu_frame.show()
        self.setMenuMode(MODE_GAMEOVER)      
            
    def setMusicVolume(self, volume):
        v=base.musicManager.getVolume()+volume*0.1
        if v>1.0:
            v=1.0
        if v<0.0:
            v=0.0    
        base.musicManager.setVolume(v)    
        self.menu_txt[MODE_MUSIC][1]='Volume: {0}'.format(int(round((v*100))))
        self.menu_buttons[1]['text']='Volume: {0}'.format(int(round((v*100))))
                     
    def setSoundVolume(self, volume):
        v=base.sfxManagerList[0].getVolume()+volume*0.1
        if v>1.0:
            v=1.0
        if v<0.0:
            v=0.0    
        base.sfxManagerList[0].setVolume(v)    
        self.menu_txt[MODE_SOUND][1]='Volume: {0}'.format(int(round((v*100))))
        self.menu_buttons[1]['text']='Volume: {0}'.format(int(round((v*100))))
         
    def setMouseSpeed(self, speed):
        speed=self.app.controler.mouseSpeed+speed
        self.app.controler.mouseSpeed=speed
        self.menu_txt[MODE_MOUSE][1]='Speed: {0}'.format(int(speed))
        self.menu_buttons[1]['text']='Speed: {0}'.format(int(speed))
        
    def showHelp(self, volume):        
        self.setMenuMode(MODE_HELP)
        self.help.show()    
        
    def startGame(self, arg=None):
        self.app.pause=False
        self.logo.hide()
        self.menu_frame.hide() 
              
    def setMenuMode(self, mode):
        self.logo.show()
        self.help.hide()
        for i in range(5):
            self.menu_buttons[i]['text']=self.menu_txt[mode][i] 
            self.menu_buttons[i].show()
            
        if mode in (MODE_GAMEOVER, MODE_HELP):
            self.menu_buttons[0].hide()
            self.menu_buttons[1].hide()
            self.menu_buttons[2].hide()
            self.menu_buttons[3].hide()
            self.logo.hide()    
        self.menu_mode=mode       
           
    def update(self, task):
        if self.shouldExit:
            self.app.exit()
        return task.cont
    
    def updateGuiNodes(self):
        wp=base.win.getProperties()
        winX = wp.getXSize()
        winY = wp.getYSize()    
            
        self.top_left.setPos(_pos2d(0,0))
        self.top_right.setPos(_pos2d(winX,0))        
        self.bottom_right.setPos(_pos2d(winX,winY))
        self.bottom_left.setPos(_pos2d(0,winY))
        self.top.setPos(_pos2d(winX/2,0))
        self.bottom.setPos(_pos2d(winX/2,winY))
        self.left.setPos(_pos2d(0,winY/2))
        self.right.setPos(_pos2d(winX,winY/2))
        self.center.setPos(_pos2d(winX/2,winY/2))    
        if winY<445:
            self.menu_buttons[0].hide()
        else:
            self.menu_buttons[0].show()
